Option Compare Binary
Option Explicit On 
Option Strict On

' N.B. Drone cells and worker cells can both be used to store food.
Public Enum ECellContents
    ccEmpty
    ccPollen
    ccNectar
    ccEgg
    ccLarva
    ccPupa
End Enum

Public Class CCell

    ' Location on the comb.
    Private mptLocation As PointF

    ' The contents of the cell.
    Private mccContents As ECellContents

    ' If this cell contains brood, what caste is the developing bee?
    Private mcstBrood As ECaste

    ' How long has it had these contents?
    ' (This is measured in ticks.)
    Private mintAge As Integer = 0

    ' Has this cell been capped?
    Private mblnIsCapped As Boolean = False

    ' Is this cell dirty? (I.e. does it need to be cleaned?)
    Private mblnIsDirty As Boolean = False

    ' Events to tell the comb about changes to the brood.
    Public Event NewEgg(ByVal cstCaste As ECaste)
    Public Event Hatch(ByVal cstCaste As ECaste, ByVal ptLocation As PointF)

    Public Sub New(ByVal ptLocation As PointF)
        MyBase.New()
        mptLocation = ptLocation
    End Sub

    Public Property Contents() As ECellContents
        Get
            Return mccContents
        End Get
        Set(ByVal Value As ECellContents)
            mccContents = Value
        End Set
    End Property

    Public Property BroodCaste() As ECaste
        Get
            Return mcstBrood
        End Get
        Set(ByVal Value As ECaste)
            mcstBrood = Value
        End Set
    End Property

    Public ReadOnly Property Location() As PointF
        Get
            Return mptLocation
        End Get
    End Property

    Public Sub LayEgg(ByVal cstCaste As ECaste)

        ' Put an egg in this cell, of the appropriate caste.
        mccContents = ECellContents.ccEgg
        mcstBrood = cstCaste
        mintAge = 0

        ' Raise an event, so that the comb can update its quantities.
        RaiseEvent NewEgg(cstCaste)

    End Sub

    Public Sub Tick()
        ' The current age (in days).
        Dim intAgeDays As Integer
        ' The length of each stage in a bee's life cycle (measured in days).
        Const cintEggDays As Integer = 4   ' 3 days as egg, 1 day hatching
        Const cintFemaleLarvaDays As Integer = 5
        Const cintDroneLarvaDays As Integer = 8
        Const cintQueenPupaDays As Integer = 7
        Const cintWorkerPupaDays As Integer = 12
        Const cintDronePupaDays As Integer = 12
        ' This stores the relevant length for this brood.
        Dim intDays As Integer

        ' If this cell contains brood, increment the age of the current contents.
        ' Then calculate the age in days.
        ' Round up, because we want to know whether we've finished a stage.
        Select Case mccContents
            Case ECellContents.ccEgg, ECellContents.ccLarva, ECellContents.ccPupa
                mintAge += 1
                intAgeDays = CType(Math.Ceiling(mintAge / CDome.TicksPerDay), Integer)
        End Select

        ' If the cell contains brood, see whether it is time for the next stage in the life-cycle.
        ' N.B. We can't combine these two Case statements together,
        ' because this one is more specific.
        Select Case mccContents
            Case ECellContents.ccEgg
                intDays = cintEggDays
                If (intAgeDays > intDays) Then
                    mccContents = ECellContents.ccLarva
                    mintAge = 0
                End If
            Case ECellContents.ccLarva
                If (mcstBrood = ECaste.cstDrone) Then
                    intDays = cintDroneLarvaDays
                Else
                    intDays = cintFemaleLarvaDays
                End If
                If (intAgeDays > intDays) Then
                    mccContents = ECellContents.ccPupa
                    mintAge = 0
                End If
            Case ECellContents.ccPupa
                Select Case mcstBrood
                    Case ECaste.cstDrone
                        intDays = cintDronePupaDays
                    Case ECaste.cstQueen
                        intDays = cintQueenPupaDays
                    Case ECaste.cstWorker
                        intDays = cintWorkerPupaDays
                End Select
                If (intAgeDays > intDays) Then
                    mccContents = ECellContents.ccEmpty
                    mblnIsDirty = True
                    mintAge = 0
                    RaiseEvent Hatch(mcstBrood, mptLocation)
                End If
        End Select

    End Sub

End Class
