Option Compare Binary
Option Explicit On 
Option Strict On

Public Class CComb

    ' TODO: Divide this into two sides?
    Private mintCellCount As Integer = 0
    Private mcelCells() As CCell

    ' The dimensions (in mm) of a standard frame (8.5" x 14").
    Private Const mcintWidth As Integer = 356
    Private Const mcintHeight As Integer = 216

    ' The location of the hive within the dome.
    Private mptHive As PointF

    ' The hive's ID.
    Private mintHiveID As Integer

    ' The bees.
    Private mintBeeCount As Integer
    Private mbeeBees() As CBee

    ' This stores the quantities, for speed of access (rather than recounting all the bees each time we need this info).
    Private mqntQuantities As SQuantities

    Public Event QuantityChanged(ByVal cstCaste As ECaste, ByVal blnBrood As Boolean, ByVal intDifference As Integer)

    Public Sub New(ByVal ptHive As PointF, ByVal intHiveID As Integer)
        MyBase.New()

        Dim celThis As CCell
        Dim intCellX As Integer
        Dim intCellY As Integer
        Dim blnOdd As Boolean = True
        Dim intIndex As Integer = 0

        ' Store the hive's position and ID.
        mptHive = ptHive
        mintHiveID = intHiveID

        ' 20 cells per odd row, 19 cells per even row. 7 pairs (odd/even) per comb.
        mintCellCount = 273
        ReDim mcelCells(mintCellCount - 1)

        ' Create new cell objects with the appropriate co-ordinates.
        ' They all share event handlers.
        For intCellY = 0 To 200 Step 15
            If blnOdd Then
                For intCellX = 0 To 350 Step 18
                    celThis = New CCell(New PointF(intCellX, intCellY))
                    AddEventHandlers(celThis)
                    mcelCells(intIndex) = celThis
                    celThis = Nothing
                    intIndex += 1
                Next intCellX
            Else
                For intCellX = 9 To 350 Step 18
                    celThis = New CCell(New PointF(intCellX, intCellY))
                    AddEventHandlers(celThis)
                    mcelCells(intIndex) = celThis
                    celThis = Nothing
                    intIndex += 1
                Next intCellX
            End If
            blnOdd = Not blnOdd
        Next intCellY

        ' Start out with no bees.
        mintBeeCount = 0
        With mqntQuantities
            .DroneCount = 0
            .BroodDroneCount = 0
            .BroodQueenCount = 0
            .BroodWorkerCount = 0
            .QueenCount = 0
            .WorkerCount = 0
        End With
        Erase mbeeBees

    End Sub

    Public Sub LoadScenario()
        Dim intIndex As Integer

        ' Have one queen, one drone, and one worker.
        mintBeeCount = 3
        With mqntQuantities
            .DroneCount = 1
            .BroodDroneCount = 0
            .BroodQueenCount = 0
            .BroodWorkerCount = 0
            .QueenCount = 1
            .WorkerCount = 1
        End With
        ReDim mbeeBees(mintBeeCount - 1)
        mbeeBees(0) = New CQueen(mptHive, New PointF(20, 30), mintHiveID)
        mbeeBees(1) = New CWorker(mptHive, New PointF(20, 50), mintHiveID)
        mbeeBees(2) = New CDrone(mptHive, New PointF(20, 10), mintHiveID)

        ' Add the event handlers for each one.
        For intIndex = 0 To mintBeeCount - 1 Step 1
            AddEventHandlers(mbeeBees(intIndex))
        Next intIndex

        ' Randomise the cell contents.
        For intIndex = 0 To mintCellCount - 1 Step 1
            Select Case Math.Floor(Rnd() * 4)
                Case 0
                    mcelCells(intIndex).LayEgg(ECaste.cstWorker)
                Case 1
                    mcelCells(intIndex).Contents = ECellContents.ccNectar
                Case 2
                    mcelCells(intIndex).Contents = ECellContents.ccPollen
                Case 3
                    mcelCells(intIndex).Contents = ECellContents.ccEmpty
            End Select
        Next intIndex

    End Sub

    ' Add the event handlers for this cell.
    Private Overloads Sub AddEventHandlers(ByVal celThis As CCell)

        AddHandler celThis.Hatch, AddressOf Hatch
        AddHandler celThis.NewEgg, AddressOf NewEgg

    End Sub

    ' Add the event handlers for this bee.
    Private Overloads Sub AddEventHandlers(ByVal beeThis As CBee)

        AddHandler beeThis.DeadBee, AddressOf DeadBee
        AddHandler beeThis.LeftHive, AddressOf LeftHive
        If (beeThis.Caste = ECaste.cstWorker) Then
            AddHandler CType(beeThis, CWorker).NewFoodSource, AddressOf AnnounceFoodSource
        End If

    End Sub

    ' Remove the event handlers for this bee.
    Private Sub RemoveEventHandlers(ByVal beeThis As CBee)

        RemoveHandler beeThis.DeadBee, AddressOf DeadBee
        RemoveHandler beeThis.LeftHive, AddressOf LeftHive
        If (beeThis.Caste = ECaste.cstWorker) Then
            RemoveHandler CType(beeThis, CWorker).NewFoodSource, AddressOf AnnounceFoodSource
        End If

    End Sub

    Public Property CellCount() As Integer
        Get
            Return mintCellCount
        End Get
        Set(ByVal Value As Integer)
            mintCellCount = Value
        End Set
    End Property

    Public Property Cells(ByVal intIndex As Integer) As CCell
        Get
            Return mcelCells(intIndex)
        End Get
        Set(ByVal Value As CCell)
            mcelCells(intIndex) = Value
        End Set
    End Property

    Public Shared ReadOnly Property Width() As Integer
        Get
            Return mcintWidth
        End Get
    End Property

    Public Shared ReadOnly Property Height() As Integer
        Get
            Return mcintHeight
        End Get
    End Property

    Public Sub Tick()
        ' This is used to loop through the cells.
        Dim intThisCell As Integer
        ' This is used to loop through the list of bees.
        Dim intThisBee As Integer

        ' Let all the cells tick.
        ' (This is to let brood develop.)
        For intThisCell = 0 To mintCellCount - 1 Step 1
            mcelCells(intThisCell).Tick()
        Next intThisCell

        ' Let each bee have a tick.
        ' N.B. The size of this array may change during the execution of the loop (e.g. if bees die),
        ' so we can't do a conventional For loop in case we wind up out of bounds.
        ' There is a risk that some bees will be overlooked, or tick twice, but I can live with that.
        ' TODO: Divide the bees into n groups, and only process each group's ticks on every nth iteration?
        intThisBee = 0
        Do While (intThisBee < mintBeeCount)
            mbeeBees(intThisBee).Tick()
            intThisBee += 1
        Loop

    End Sub

    Private Sub Hatch(ByVal cstCaste As ECaste, ByVal ptCell As PointF)

        ' Make room for a new bee in the array.
        ReDim Preserve mbeeBees(mintBeeCount)

        ' Create a new mature bee, of the relevant caste,
        ' and store a reference to it in the array.
        Select Case cstCaste
            Case ECaste.cstDrone
                mbeeBees(mintBeeCount) = New CDrone(mptHive, ptCell, mintHiveID)
            Case ECaste.cstQueen
                mbeeBees(mintBeeCount) = New CQueen(mptHive, ptCell, mintHiveID)
            Case ECaste.cstWorker
                mbeeBees(mintBeeCount) = New CWorker(mptHive, ptCell, mintHiveID)
        End Select

        ' Add the event handlers.
        AddEventHandlers(mbeeBees(mintBeeCount))

        ' Increment the number of bees.
        mintBeeCount += 1

        ' Increment the relevant mature quantity.
        CShared.QuantityChanged(cstCaste, False, 1, mqntQuantities)

        ' Signal this change to the hive.
        RaiseEvent QuantityChanged(cstCaste, False, 1)

        ' Similarly, decrement the relevant brood quantity.
        CShared.QuantityChanged(cstCaste, True, -1, mqntQuantities)

        ' Signal this change to the hive.
        RaiseEvent QuantityChanged(cstCaste, True, -1)

    End Sub

    Private Sub NewEgg(ByVal cstCaste As ECaste)

        ' Increment the relevant quantity.
        CShared.QuantityChanged(cstCaste, True, 1, mqntQuantities)

        ' Signal this change to the hive.
        RaiseEvent QuantityChanged(cstCaste, True, 1)

    End Sub

    Public Sub GetQuantities(ByRef qntQuantities As SQuantities)

        ' Return the quantities.
        ' N.B. Since this is a structure rather than a class,
        ' we can just copy the values straight across.
        qntQuantities = mqntQuantities

    End Sub

    Public Sub AssignTimeSlice(ByVal tsThis As CTimeSlice)
        ' This is used to loop through the list of bees.
        Dim intThisBee As Integer
        ' This accesses each bee in turn.
        Dim beeThis As SBee

        ' Comb.
        tsThis.Comb = Me

        ' Bees.
        ' N.B. The time-slice contains structures rather than objects,
        ' so we are copying around the contents rather than the reference.
        tsThis.BeeCount = mintBeeCount
        For intThisBee = 0 To mintBeeCount - 1 Step 1
            With mbeeBees(intThisBee)
                beeThis.Caste = .Caste
                beeThis.Direction = .Direction
                beeThis.Location = .Location
                beeThis.Motion = .Motion
            End With
            tsThis.Bees(intThisBee) = beeThis
        Next intThisBee

    End Sub

    ' This handles the event when a bee dies.
    Public Sub DeadBee(ByVal beeThis As CBee)

        ' TODO: The dead bee turns into a corpse that must be removed.

        ' Remove this bee.
        RemoveBee(beeThis)

    End Sub

    ' This handles the event when a bee leaves the hive.
    Public Sub LeftHive(ByVal beeThis As CBee)

        ' Remove this bee.
        RemoveBee(beeThis)

    End Sub

    Private Sub RemoveBee(ByVal beeThis As CBee)

        ' Remove this bee from the array.
        CShared.RemoveBee(beeThis, mintBeeCount, mbeeBees)

        ' Clean up.
        RemoveBeeCleanup(beeThis)

    End Sub

    ' Handle the clean-up after removing a bee.
    Private Sub RemoveBeeCleanup(ByVal beeThis As CBee)

        ' Decrement the relevant quantity.
        CShared.QuantityChanged(beeThis.Caste, False, -1, mqntQuantities)

        ' Signal this change to the hive.
        RaiseEvent QuantityChanged(beeThis.Caste, False, -1)

        ' Remove the event handlers.
        RemoveEventHandlers(beeThis)

    End Sub

    ' If the queen is on this comb, kill it, and return True.
    ' If the queen isn't here, return False.
    Public Function KillQueen() As Boolean
        ' This stores the function's return value.
        Dim blnReturnValue As Boolean = False
        ' This is used to loop through the list of bees.
        Dim intThisBee As Integer
        ' This accesses the bee to delete.
        Dim beeThis As CBee

        ' Loop through the list of bees looking for the queen.
        ' (Stop if we find it, for efficiency.)
        intThisBee = 0
        Do While (intThisBee < mintBeeCount) And Not blnReturnValue
            If (mbeeBees(intThisBee).Caste = ECaste.cstQueen) Then
                blnReturnValue = True
            Else
                intThisBee += 1
            End If
        Loop

        ' If we found it, delete it, and do the appropriate clean-up.
        If blnReturnValue Then
            beeThis = mbeeBees(intThisBee)
            CShared.RemoveBee(intThisBee, mintBeeCount, mbeeBees)
            RemoveBeeCleanup(beeThis)
            beeThis = Nothing
        End If

        ' Return the appropriate result from this function.
        Return blnReturnValue

    End Function

    ' This method is called when one bee is announcing a new food source to other bees.
    Public Sub AnnounceFoodSource(ByVal ptLocation As PointF, ByVal dblScore As Double)
        ' This is used to loop through the list of bees.
        Dim intThisBee As Integer
        ' This accesses each worker in turn.
        ' N.B. We need a worker object, not a bee object, to access caste-specific methods.
        Dim wrkThis As CWorker

        ' Loop through the list of bees on this comb.
        intThisBee = 0
        Do While (intThisBee < mintBeeCount)

            ' See whether this bee is a worker.
            ' (If not, ignore it.)
            If (mbeeBees(intThisBee).Caste = ECaste.cstWorker) Then

                ' If so, tell it about this food source.
                wrkThis = CType(mbeeBees(intThisBee), CWorker)
                wrkThis.AnnounceFoodSource(ptLocation, dblScore)
                wrkThis = Nothing

            End If  ' end "is this bee a worker?" if statement.

            ' Move to the next bee.
            intThisBee += 1

            ' Finish the loop through the list of bees.
        Loop

    End Sub

    ' This is used when a bee enters a hive, and comes under the comb's jurisdiction.
    Public Sub AddBee(ByVal beeThis As CBee)

        ' Make room for this bee in the array.
        ReDim Preserve mbeeBees(mintBeeCount)

        ' Store a reference to this bee in the array.
        mbeeBees(mintBeeCount) = beeThis

        ' Increment the number of bees.
        mintBeeCount += 1

        ' Increment the relevant quantity.
        CShared.QuantityChanged(beeThis.Caste, False, 1, mqntQuantities)

        ' Signal this change to the hive.
        RaiseEvent QuantityChanged(beeThis.Caste, False, 1)

        ' Add the event handlers for this bee.
        AddEventHandlers(beeThis)

    End Sub

End Class
