Option Compare Binary
Option Explicit On 
Option Strict On

Public Class CHive

    ' The ID.
    Private mintID As Integer

    ' The location, in dome co-ordinates.
    Private mptLocation As PointF

    ' This stores the quantities, for speed of access (rather than recounting all the bees each time we need this info).
    Private mqntQuantities As SQuantities

    Public Event QuantityChanged(ByVal cstCaste As ECaste, ByVal blnBrood As Boolean, ByVal intDifference As Integer)

    ' The combs.
    Private mintCombCount As Integer = 0
    Private mcmbCombs() As CComb

    Public Sub GetQuantities(ByRef qntQuantities As SQuantities)

        ' Return the quantities.
        ' N.B. Since this is a structure rather than a class,
        ' we can just copy the values straight across.
        qntQuantities = mqntQuantities

    End Sub

    Public Function Combs(ByVal intIndex As Integer) As CComb
        Return mcmbCombs(intIndex)
    End Function

    Public Sub New(ByVal ptLocation As PointF, ByVal intID As Integer)
        MyBase.New()

        ' This stores each comb's quantities.
        Dim qntComb As SQuantities

        ' Record this location.
        mptLocation = ptLocation

        ' Assign the ID.
        mintID = intID

        ' Set all the starting quantities to zero.
        With mqntQuantities
            .DroneCount = 0
            .BroodDroneCount = 0
            .BroodQueenCount = 0
            .BroodWorkerCount = 0
            .QueenCount = 0
            .WorkerCount = 0
        End With

        ' We have 10 combs per hive.
        mintCombCount = 10
        ReDim mcmbCombs(mintCombCount - 1)
        Dim intThisComb As Integer
        For intThisComb = 0 To mintCombCount - 1 Step 1
            mcmbCombs(intThisComb) = New CComb(mptLocation, mintID)
        Next intThisComb

        ' Load a scenario for the first comb.
        mcmbCombs(0).LoadScenario()

        ' Get each comb's quantities, and add them to the dome's total,
        ' then add an event handler for changes to these quantities.
        For intThisComb = 0 To mintCombCount - 1 Step 1
            mcmbCombs(intThisComb).GetQuantities(qntComb)
            With mqntQuantities
                .DroneCount += qntComb.DroneCount
                .BroodDroneCount += qntComb.BroodDroneCount
                .BroodQueenCount += qntComb.BroodQueenCount
                .BroodWorkerCount += qntComb.BroodWorkerCount
                .QueenCount += qntComb.QueenCount
                .WorkerCount += qntComb.WorkerCount
            End With
            AddHandler mcmbCombs(intThisComb).QuantityChanged, AddressOf CombQuantityChanged
        Next intThisComb

    End Sub

    Public Sub Tick()
        ' This is used to loop through the list of combs.
        Dim intThisComb As Integer

        ' Let each comb have a tick.
        For intThisComb = 0 To mintCombCount - 1 Step 1
            mcmbCombs(intThisComb).Tick()
        Next intThisComb

    End Sub

    Public ReadOnly Property Location() As PointF
        Get
            Return mptLocation
        End Get
    End Property

    Public Sub AssignTimeSlice(ByVal tsThis As CTimeSlice, ByVal intFrameIndex As Integer)

        ' Hive view.
        tsThis.DomeView = False

        ' Comb.
        mcmbCombs(intFrameIndex).AssignTimeSlice(tsThis)

    End Sub

    Private Sub CombQuantityChanged(ByVal cstCaste As ECaste, ByVal blnBrood As Boolean, ByVal intDifference As Integer)

        ' Update the relevant quantity.
        CShared.QuantityChanged(cstCaste, blnBrood, intDifference, mqntQuantities)

        ' Raise an event to show that the hive's quantity has changed.
        RaiseEvent QuantityChanged(cstCaste, blnBrood, intDifference)

    End Sub

    ' This method is called when a bee disappears outside the hive.
    ' This would usually happen when the bee dies,
    ' but would also apply if a drone gets lost and is "adopted" by another hive.
    ' Either way, the total number of bees associated with this hive needs to be reduced.
    Public Sub LostBee(ByVal cstCaste As ECaste, ByVal blnBrood As Boolean, ByVal intDifference As Integer)

        ' Update the relevant quantity.
        CShared.QuantityChanged(cstCaste, blnBrood, intDifference, mqntQuantities)

        ' Don't raise an event, since the dome already knows whatever it needs to.

    End Sub

    ' Try to kill this hive's queen - return True if we succeed, False if we fail.
    ' N.B. There is a potential problem here, if the queen moves around while we're trying to kill her.
    ' E.g. If we search comb 1 while she's on comb 2, then she moves to comb 1 while we're searching comb 2.
    ' Hopefully this problem won't occur - I don't see any easy workaround for it.
    ' If all else fails, the user can always try again - this is an intervention rather than a normal part of the simulation.
    Public Function KillQueen() As Boolean
        ' This stores the function's return value.
        Dim blnReturnValue As Boolean = False
        ' This is used to loop through the list of combs.
        Dim intThisComb As Integer

        ' Loop through the combs, but stop if we've killed the queen (for efficiency).
        ' (We assume that there is only one queen per hive.)
        intThisComb = 0
        Do While (intThisComb < mintCombCount) And Not blnReturnValue
            If mcmbCombs(intThisComb).KillQueen() Then
                blnReturnValue = True
            Else
                intThisComb += 1
            End If
        Loop

        ' If we haven't found the queen in the hive, check the outdoor bees in the dome.
        ' (It's very unusual for the queen to fly around outside, but it is possible.)
        If Not blnReturnValue Then
            blnReturnValue = CDome.KillQueen(mintID)
        End If

        ' Return the appropriate result from this function.
        Return blnReturnValue

    End Function

    Public ReadOnly Property ID() As Integer
        Get
            Return mintID
        End Get
    End Property

End Class
