' This stores the quantities of different bee castes/ages.
' Brood is a collective term for all immature life stages: egg/pupa/larva.
Public Structure SQuantities
    Dim QueenCount As Integer
    Dim WorkerCount As Integer
    Dim DroneCount As Integer
    Dim BroodQueenCount As Integer
    Dim BroodWorkerCount As Integer
    Dim BroodDroneCount As Integer
End Structure

' The seasons of the year.
Public Enum ESeason
    ssnSpring
    ssnSummer
    ssnAutumn
    ssnWinter
End Enum

Public Class CShared

    ' Update a quantity.
    Public Shared Sub QuantityChanged(ByVal cstCaste As ECaste, ByVal blnBrood As Boolean, ByVal intDifference As Integer, ByRef qntQuantities As SQuantities)

        Select Case cstCaste
            Case ECaste.cstDrone
                If blnBrood Then
                    qntQuantities.BroodDroneCount += intDifference
                Else
                    qntQuantities.DroneCount += intDifference
                End If
            Case ECaste.cstQueen
                If blnBrood Then
                    qntQuantities.BroodQueenCount += intDifference
                Else
                    qntQuantities.QueenCount += intDifference
                End If
            Case ECaste.cstWorker
                If blnBrood Then
                    qntQuantities.BroodWorkerCount += intDifference
                Else
                    qntQuantities.WorkerCount += intDifference
                End If
        End Select

    End Sub

    Public Overloads Shared Sub RemoveBee(ByVal beeThis As CBee, ByRef intBeeCount As Integer, ByRef beeBees As CBee())
        ' This is used to loop through the list of bees.
        Dim intThisBee As Integer
        ' This stores the relevant bee's index in the array.
        Dim intIndex As Integer = -1

        ' Look for this bee in the array.
        ' (Stop looping if we find it, for efficiency.)
        intThisBee = 0
        Do While (intThisBee < intBeeCount) And (intIndex = -1)
            If (beeBees(intThisBee) Is beeThis) Then
                intIndex = intThisBee
            End If
            intThisBee += 1
        Loop

        ' See whether we found it.
        ' (If not, don't do anything else.)
        If (intIndex > -1) Then

            ' If so, remove it.
            RemoveBee(intIndex, intBeeCount, beeBees)

        End If  ' end "did we find this bee?" if statement.

    End Sub

    Public Overloads Shared Sub RemoveBee(ByVal intIndex As Integer, ByRef intBeeCount As Integer, ByRef beeBees As CBee())
        ' This is used to loop through the list of bees.
        Dim intThisBee As Integer

        ' Shuffle all the subsequent bees down the array (towards the start).
        For intThisBee = intIndex To intBeeCount - 2 Step 1
            beeBees(intThisBee) = beeBees(intThisBee + 1)
        Next intThisBee

        ' Free up the reference to the final bee.
        beeBees(intBeeCount - 1) = Nothing

        ' Resize the array.
        If (intBeeCount > 1) Then
            ReDim Preserve beeBees(intBeeCount - 2)
        Else
            Erase beeBees
        End If

        ' Decrement the number of bees.
        intBeeCount -= 1

    End Sub

End Class
