Option Compare Binary
Option Explicit On 
Option Strict On

' A simplified version of a bee.
Public Structure SBee
    Dim Location As PointF
    Dim Direction As Double
    Dim Motion As EMotion
    Dim Caste As ECaste
End Structure

Public Class CTimeSlice

    ' This class stores a snapshot of the display information at a moment in time.
    ' With an array of snapshots, you can then record/play a video.

    ' Each time slice either represents the outside of the hive,
    ' or one side of a comb inside the hive.
    Private mblnDomeView As Boolean

    ' The date/time in the simulation.
    Private mdtmSimulation As DateTime

    ' The dome's temperature (in degrees Kelvin).
    Private mintTemperature As Integer

    ' The quantities.
    ' N.B. This is distinct from the number of bees below;
    ' this stores the total number of bees in the dome,
    ' rather than the number which are being displayed in this zone.
    Private mqntQuantities As SQuantities

    ' Hives.
    Private mintHiveCount As Integer
    Private mptHives As PointF()

    ' Food sources.
    Private mintFSCount As Integer
    Private mptFoodSources As PointF()

    ' Bees.
    Private mintBeeCount As Integer
    Private mbeeBees As SBee()

    ' Comb.
    Private mcmbComb As CComb

    Public Property BeeCount() As Integer
        Get
            Return mintBeeCount
        End Get
        Set(ByVal Value As Integer)
            mintBeeCount = Value
            If (mintBeeCount > 0) Then
                ReDim mbeeBees(mintBeeCount - 1)
            Else
                Erase mbeeBees
            End If
        End Set
    End Property

    Public Property Bees(ByVal intIndex As Integer) As SBee
        Get
            Return mbeeBees(intIndex)
        End Get
        Set(ByVal Value As SBee)
            mbeeBees(intIndex) = Value
        End Set
    End Property

    Public Property Comb() As CComb
        Get
            Return mcmbComb
        End Get
        Set(ByVal Value As CComb)
            mcmbComb = Value
        End Set
    End Property

    Public Property DomeView() As Boolean
        Get
            Return mblnDomeView
        End Get
        Set(ByVal Value As Boolean)
            mblnDomeView = Value
        End Set
    End Property

    Public Property FoodSourceCount() As Integer
        Get
            Return mintFSCount
        End Get
        Set(ByVal Value As Integer)
            mintFSCount = Value
            If (mintFSCount > 0) Then
                ReDim mptFoodSources(mintFSCount - 1)
            Else
                Erase mptFoodSources
            End If
        End Set
    End Property

    Public Property FoodSources(ByVal intIndex As Integer) As PointF
        Get
            Return mptFoodSources(intIndex)
        End Get
        Set(ByVal Value As PointF)
            mptFoodSources(intIndex) = Value
        End Set
    End Property

    Public Property HiveCount() As Integer
        Get
            Return mintHiveCount
        End Get
        Set(ByVal Value As Integer)
            mintHiveCount = Value
            If (mintHiveCount > 0) Then
                ReDim mptHives(mintHiveCount - 1)
            Else
                Erase mptHives
            End If
        End Set
    End Property

    Public Property Hives(ByVal intIndex As Integer) As PointF
        Get
            Return mptHives(intIndex)
        End Get
        Set(ByVal Value As PointF)
            mptHives(intIndex) = Value
        End Set
    End Property

    Public Property Quantities() As SQuantities
        Get
            Return mqntQuantities
        End Get
        Set(ByVal Value As SQuantities)
            mqntQuantities = Value
        End Set
    End Property

    Public Property SimulatedDateTime() As Date
        Get
            Return mdtmSimulation
        End Get
        Set(ByVal Value As Date)
            mdtmSimulation = Value
        End Set
    End Property

    ' The temperature in degrees Kelvin.
    Public Property TemperatureKelvin() As Integer
        Get
            Return mintTemperature
        End Get
        Set(ByVal Value As Integer)
            mintTemperature = Value
        End Set
    End Property

    ' The temperature in degrees Celsius.
    Public ReadOnly Property TemperatureCelsius() As Integer
        Get
            Return mintTemperature - 273
        End Get
    End Property

End Class
