Option Compare Binary
Option Explicit On 
Option Strict On

Public Class CWorker
    Inherits CFemale

    Private Enum ERole
        rolForager
        rolScout
    End Enum

    ' TODO: Extend this to handle other roles, e.g. guard bee.
    Private mrolRole As ERole

    ' Has this bee found any food?
    Private mblnFoundFood As Boolean = False

    ' The most recently located food source.
    Private mptFoodSource As PointF

    ' The score for that food source.
    Private mdblFoodSourceScore As Double = 0

    Public Event NewFoodSource(ByVal ptLocation As PointF, ByVal dblScore As Double)

    Public Sub New(ByVal ptHive As PointF, ByVal ptBee As PointF, ByVal intHiveID As Integer, Optional ByVal blnOutside As Boolean = False)
        MyBase.New(ptHive, ptBee, intHiveID, blnOutside)
        ' mdblDirection = -Math.PI / 2
        ' Choose a role.
        Select Case Math.Floor(Rnd() * 2)
            Case 0
                mrolRole = ERole.rolForager
            Case 1
                mrolRole = ERole.rolScout
        End Select
    End Sub

    Public Overrides Sub Tick()
        MyBase.Tick()

        ' TODO: Change role if appropriate. Or do that when entering/leaving hive?

        ' Handle a bee that is sitting still.
        If (mmotThis = EMotion.motStill) Then
            If mblnOutside Then
                mmotThis = EMotion.motSearching
            Else
                ' If a forager hasn't found food, wait for a scout to come back.
                ' If a scout hasn't found food, go and look for some.
                ' If a scout or forager has found food, wait around a little while
                ' in case of better news from another scout.
                Select Case mrolRole
                    Case ERole.rolScout
                        If mblnFoundFood Then
                            If (Rnd() < 0.2) Then
                                LeaveHive()
                            End If
                        Else
                            LeaveHive()
                        End If
                    Case ERole.rolForager
                        If mblnFoundFood Then
                            If (Rnd() < 0.2) Then
                                LeaveHive()
                            End If
                        End If
                End Select
            End If
        End If
    End Sub

    Public Overrides ReadOnly Property Caste() As ECaste
        Get
            Return ECaste.cstWorker
        End Get
    End Property

    Public Overrides ReadOnly Property LifeExpectancy() As Integer
        Get
            ' 8 months (August->April).
            Return 240
        End Get
    End Property

    ' This method is called when another bee is announcing a new food source.
    Public Sub AnnounceFoodSource(ByVal ptLocation As PointF, ByVal dblScore As Double)

        ' See whether this food source is "worth getting out of bed for".
        ' (If it's too poor, the bee would rather stay in the hive doing nothing than go to it.)
        If (dblScore > 0.5) Then

            ' Compare this to the currently favoured food source, and decide whether to defect.
            If (Not mblnFoundFood) Or (dblScore > mdblFoodSourceScore) Then
                mblnFoundFood = True
                mptFoodSource = ptLocation
                mdblFoodSourceScore = dblScore
                LeaveHive()
            End If

        End If  ' end "reasonable source?" if statement.

    End Sub

    Protected Overrides Sub LeaveHive()
        MyBase.LeaveHive()

        ' Don't go anywhere if it's too cold outside.
        If (CDome.TemperatureKelvin >= mcintMinTemperature) Then

            Select Case mrolRole
                Case ERole.rolForager, ERole.rolScout
                    ' If the bee found food, it should go straight to the known food source.
                    If mblnFoundFood Then
                        mmotThis = EMotion.motDefinite
                        mptTarget = mptFoodSource
                    Else
                        mmotThis = EMotion.motSearching
                    End If
            End Select

        End If  ' end "warm enough?" if statement.

    End Sub

    Protected Overrides Sub EnterHive(ByVal hvThis As CHive)
        MyBase.EnterHive(hvThis)

        ' Tell the other bees here about a new food source.
        If (mrolRole = ERole.rolScout) Then
            If mblnFoundFood Then
                RaiseEvent NewFoodSource(mptFoodSource, mdblFoodSourceScore)
            End If
        End If

    End Sub

    Private Function GetFoodSource(ByVal ptLocation As PointF) As CFoodSource
        ' This is used to loop through the list of food sources.
        Dim intThisFoodSource As Integer
        ' This accesses each food source in turn.
        Dim fsThis As CFoodSource
        ' This accesses the function's return value.
        Dim fsReturnValue As CFoodSource

        For intThisFoodSource = 0 To CDome.FoodSourceCount - 1 Step 1
            fsThis = CDome.FoodSources(intThisFoodSource)
            If ((fsThis.Location.X = ptLocation.X) And (fsThis.Location.Y = ptLocation.Y)) Then
                fsReturnValue = fsThis
            End If
            fsThis = Nothing
        Next intThisFoodSource

        ' Return the food source as the function's result.
        Return fsReturnValue

    End Function

    Private Function NearFoodSource(ByRef ptLocation As PointF) As Boolean
        ' This stores the function's return value.
        Dim blnReturnValue As Boolean = False
        ' This is used to loop through the list of food sources.
        Dim intThisFoodSource As Integer
        ' This accesses each food source in turn.
        Dim fsThis As CFoodSource
        ' Arbitrary choice - the bee can detect a food source from 50m away.
        Const cintRange As Integer = 50

        For intThisFoodSource = 0 To CDome.FoodSourceCount - 1 Step 1
            fsThis = CDome.FoodSources(intThisFoodSource)
            If ((mptBee.X >= fsThis.Location.X - cintRange) _
            And (mptBee.X <= fsThis.Location.X + cintRange) _
            And (mptBee.Y >= fsThis.Location.Y - cintRange) _
            And (mptBee.Y <= fsThis.Location.Y + cintRange)) Then
                ptLocation = fsThis.Location
                blnReturnValue = True
            End If
            fsThis = Nothing
        Next intThisFoodSource

        ' Return the appropriate result from this function.
        Return blnReturnValue

    End Function

    ' Search for food.
    Protected Overrides Sub Search()
        ' This stores the location of a food source.
        Dim ptFood As PointF
        ' Arbitrary choice - don't go more than 300m from the hive.
        Const cintRange As Integer = 300

        ' If the bee is within sight (smell?) of a food source, go straight towards it.
        If NearFoodSource(ptFood) Then
            mmotThis = EMotion.motDefinite
            mptTarget = ptFood
            ' If it gets too far away from the hive, head for home.
        ElseIf (mptBee.X < -cintRange) Or (mptBee.X > cintRange) _
            Or (mptBee.Y < -cintRange) Or (mptBee.Y > cintRange) Then
            mblnFoundFood = False
            mmotThis = EMotion.motHoming
            mptTarget = mptHive
        End If

    End Sub

    Protected Overrides Sub ReachTarget()
        ' This accesses the food source.
        Dim fsThis As CFoodSource

        ' When a worker reaches a food source, it should start homing for the hive.
        ' N.B. We use the target location, rather than the bee's current position,
        ' since this method gets called when the bee is "close enough" to the target.
        mblnFoundFood = True
        mptFoodSource = mptTarget
        fsThis = GetFoodSource(mptFoodSource)
        mdblFoodSourceScore = fsThis.Score
        fsThis = Nothing
        mmotThis = EMotion.motHoming
        mptTarget = mptHive

        ' Eat while here, to avoid starvation.
        Eat()

    End Sub

End Class
